// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "cache.h"

#include <shared/clientinfo.h>

template class Core::Cache<Shared::ClientInfo>;

template <class T>
Core::Cache<T>::Cache(const QString& name):
    storage(name),
    cache(new std::map<QString, T> ()) {}

template <class T>
Core::Cache<T>::~Cache() {
    close();
    delete cache;
}

template <class T>
void Core::Cache<T>::open() {
    storage.open();}

template <class T>
void Core::Cache<T>::close() {
    storage.close();}

template <class T>
void Core::Cache<T>::addRecord(const QString& key, const T& value) {
    storage.addRecord(key, value);
    cache->insert(std::make_pair(key, value));
}

template <class T>
T Core::Cache<T>::getRecord(const QString& key) const {
    typename std::map<QString, T>::const_iterator itr = cache->find(key);
    if (itr == cache->end()) {
        T value = storage.getRecord(key);
        itr = cache->insert(std::make_pair(key, value)).first;
    }

    return itr->second;
}

template<typename T>
void Core::Cache<T>::changeRecord(const QString& key, const T& value) {
    storage.changeRecord(key, value);
    cache->at(key) = value;
}

template<typename T>
void Core::Cache<T>::removeRecord(const QString& key) {
    storage.removeRecord(key);
    cache->erase(key);
}
